//+------------------------------------------------------------------+
//|                                                       vortex.mq4 |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  Orange
#property indicator_color2  LimeGreen
#property indicator_width1  2
#property indicator_width2  2

//
//
//
//
//

extern string TimeFrame      = "current time frame";
extern int    VortexPeriod   = 14;
extern bool   ShowDifference = false;
extern bool   Interpolate    = true;

//
//
//
//
//

double vopBuffer[];
double vomBuffer[];
double vmpBuffer[];
double vmmBuffer[];
double rngBuffer[];
double difBuffer[];

//
//
//
//
//

string IndicatorFileName;
int    timeFrame;
bool   calculateVor;
bool   returningBars;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(1,vomBuffer);
   SetIndexBuffer(2,vmpBuffer);
   SetIndexBuffer(3,vmmBuffer);
   SetIndexBuffer(4,rngBuffer);
   if (ShowDifference)
   {
      SetIndexBuffer(0,difBuffer);
      SetIndexBuffer(5,vopBuffer);
         SetIndexStyle(0,DRAW_HISTOGRAM);
         SetIndexStyle(1,DRAW_NONE);
   }
   else      
   {
      SetIndexBuffer(0,vopBuffer);
      SetIndexBuffer(5,difBuffer);
         SetIndexStyle(0,DRAW_LINE);
         SetIndexStyle(1,DRAW_LINE);
   }
   
      //
      //
      //
      //
      //
      
         IndicatorFileName = WindowExpertName();
         calculateVor      = (TimeFrame=="calculateVor");
         returningBars     = (TimeFrame=="returnBars");
         timeFrame         = stringToTimeFrame(TimeFrame);

      //
      //
      //
      //
      //
         
   IndicatorShortName("Vortex "+timeFrameToString(timeFrame)+" ( "+VortexPeriod+")");         
   return(0);
}

int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//


int start()
{
   int counted_bars=IndicatorCounted();
   int limit,i;

   if(counted_bars < 0)   return(-1);
   if(counted_bars > 0)   counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-2);
               if (returningBars)  { vomBuffer[0] = limit; return(0); }
           
   //
   //
   //
   //
   //

   if (calculateVor || timeFrame==Period())
   {
      for(i = limit; i >= 0; i--)
      {
         rngBuffer[i] = MathMax(High[i],Close[i+1])-MathMin(Low[i],Close[i+1]);
         vmpBuffer[i] = MathAbs(High[i] - Low[i+1]);
         vmmBuffer[i] = MathAbs(Low[i] - High[i+1]);
         vopBuffer[i] = EMPTY_VALUE;
         vomBuffer[i] = EMPTY_VALUE;

         if ((Bars-i)<VortexPeriod) continue;
         
         //
         //
         //
         //
         //
         
            double vmpSum = 0;
            double vmmSum = 0;
            double rngSum = 0;
            for (int k=0; k<VortexPeriod; k++)
            {
               vmpSum += vmpBuffer[i+k];
               vmmSum += vmmBuffer[i+k];
               rngSum += rngBuffer[i+k];
            }
            if (rngSum !=0)
            {
               vopBuffer[i] = vmpSum/rngSum;
               vomBuffer[i] = vmmSum/rngSum;
            }
            difBuffer[i] = vopBuffer[i]-vomBuffer[i];
      }
      return(0);
   }         
   
   //
   //
   //
   //
   //
   
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         if (ShowDifference)
                difBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateVor",VortexPeriod,ShowDifference,0,y);
         else { vomBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateVor",VortexPeriod,ShowDifference,0,y);
                vopBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateVor",VortexPeriod,ShowDifference,1,y);
         }               

         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(k = 1; k < n; k++)
            {
               if (ShowDifference)
                     difBuffer[i+k]  = k*factor*difBuffer[i+n]  + (1.0-k*factor)*difBuffer[i];
               else  
                  {
                     vomBuffer[i+k]  = k*factor*vomBuffer[i+n]  + (1.0-k*factor)*vomBuffer[i];
                     vopBuffer[i+k]  = k*factor*vopBuffer[i+n]  + (1.0-k*factor)*vopBuffer[i];
                  }                
            }               
   }

   //
   //
   //
   //
   //

   return(0);      
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}